#include <Foundation/Foundation.h>
#include <EOAccess/EOModel.h>
#include <EOAccess/EOEntity.h>
#include <EOAccess/EOAttribute.h>
#include <EOControl/EOGenericRecord.h>
#include <EOAccess/EOJoin.h>
#include <EOAccess/EORelationship.h>
#include <EOAccess/EOModelGroup.h>
#include <EOControl/EOEditingContext.h>
#include <EOControl/EOFetchSpecification.h>

void gdl2test()
{
	NSMutableArray *empProps;
	NSMutableArray *depProps;
	NSMutableArray *keys;

	EOModel *myModel = [EOModel new];
	[myModel setName: @"example"]; // each model needs an internal name
	[myModel setAdaptorName: @"Postgres95"]; // the only one supported now

	NSDictionary *myConnectionDictionary =
		[NSDictionary dictionaryWithObjectsAndKeys:
		@"localhost", @"databaseServer",
		@"enterprise", @"databaseName",
		@"yoozer", @"userName",
		@"topzekret", @"password",
		@"%@_sq", @"primaryKeySequenceNameFormat",
		nil];
	[myModel setConnectionDictionary: myConnectionDictionary];

	EOEntity *empEntity = [EOEntity new];
	[empEntity setName: @"Employees"]; //name of the entity
	[empEntity setExternalName: @"emp"]; //name of database table
	[empEntity setClassName: @"EOGenericRecord"]; //default if nil used
	[myModel addEntity: empEntity];

	empProps = [NSMutableArray new];
	keys = [NSMutableArray new];

	EOAttribute *myAttribute = [EOAttribute new];
	[myAttribute setName: @"empNumber"]; //name of the attribute
	[myAttribute setColumnName: @"empno"]; //name of table column
	[myAttribute setExternalType: @"int4"]; // database type
	[myAttribute setValueClassName: @"NSNumber"]; // mapped to GS type
	[myAttribute setValueType: @"i"]; // int aspect of NSNumber
	[myAttribute setAllowsNull: NO];
	[empEntity addAttribute: myAttribute];
	[keys addObject: myAttribute];
	RELEASE(myAttribute);

	myAttribute = [EOAttribute new];
	[myAttribute setName: @"name"];
	[myAttribute setColumnName: @"name"];
	[myAttribute setExternalType: @"varchar"];
	[myAttribute setValueClassName: @"NSString"];
	[myAttribute setWidth: 40];
	[myAttribute setAllowsNull: YES];
	[empEntity addAttribute: myAttribute];
	[empProps addObject: myAttribute];
	RELEASE(myAttribute);

	myAttribute = [EOAttribute new];
	[myAttribute setName: @"department"];
	[myAttribute setColumnName: @"dep"];
	[myAttribute setExternalType: @"int4"];
	[myAttribute setValueClassName: @"NSNumber"];
	[myAttribute setValueType: @"i"];
	[myAttribute setAllowsNull: YES];
	[empEntity addAttribute: myAttribute];

	myAttribute = [EOAttribute new];
	[myAttribute setName: @"salary"];
	[myAttribute setColumnName: @"salary"];
	[myAttribute setExternalType: @"int4"];
	[myAttribute setValueClassName: @"NSNumber"];
	[myAttribute setValueType: @"i"];
	[myAttribute setAllowsNull: YES];
	[empEntity addAttribute: myAttribute];
	[empProps addObject: myAttribute];
	RELEASE(myAttribute);

	if (![empEntity setPrimaryKeyAttributes: keys])
	{
		NSLog(@"error in assigning primary keys to entity %s", [empEntity name]);
		RELEASE(keys);
		RELEASE(empProps);
		RELEASE(empEntity);
		RELEASE(myModel);
		return;
	}

	RELEASE(keys);

	depProps = [NSMutableArray new];
	keys = [NSMutableArray new];

	EOEntity *depEntity = [EOEntity new];
	[depEntity setName: @"Departments"]; //name of the entity
	[depEntity setExternalName: @"dep"]; //name of database table
	[depEntity setClassName: @"EOGenericRecord"]; //default if nil used
	[myModel addEntity: depEntity];

	myAttribute = [EOAttribute new];
	[myAttribute setName: @"depNumber"];
	[myAttribute setColumnName: @"depno"];
	[myAttribute setExternalType: @"int4"];
	[myAttribute setValueClassName: @"NSNumber"];
	[myAttribute setAllowsNull: NO];
	[depEntity addAttribute: myAttribute];
	[keys addObject: myAttribute];
	RELEASE(myAttribute);

	myAttribute = [EOAttribute new];
	[myAttribute setName: @"name"];
	[myAttribute setColumnName: @"name"];
	[myAttribute setExternalType: @"varchar"];
	[myAttribute setWidth: 40];
	[myAttribute setValueClassName: @"NSString"];
	[myAttribute setAllowsNull: YES]; // default
	[depEntity addAttribute: myAttribute];
	[depProps addObject: myAttribute];
	RELEASE(myAttribute);

	[depEntity setClassProperties: depProps];

	if (![depEntity setPrimaryKeyAttributes: keys])
	{
		NSLog(@"error in assigning primary keys to entity %s", [depEntity name]);
		RELEASE(keys);
		RELEASE(empProps);
		RELEASE(depProps);
		RELEASE(empEntity);
		RELEASE(depEntity);
		RELEASE(myModel);
		return;
	}

	RELEASE(keys);

	EOJoin *myJoin = [[EOJoin alloc]
		initWithSourceAttribute: [empEntity attributeNamed: @"department"]
		destinationAttribute: [depEntity attributeNamed: @"depNumber"]];

	EORelationship *myRelationship = [EORelationship new];
	[myRelationship setName: @"toDepartment"];
	[empEntity addRelationship: myRelationship];
	[myRelationship addJoin: myJoin];
	[myRelationship setJoinSemantic: EOLeftOuterJoin];
	[myRelationship setToMany: NO];

	RELEASE(myJoin);

	[empProps addObject: myRelationship];
	[empEntity setClassProperties: empProps];

	RELEASE(myRelationship);
	RELEASE(empProps);
	RELEASE(empEntity);

	myJoin = [[EOJoin alloc]
		initWithSourceAttribute: [depEntity attributeNamed: @"depNumber"]
		destinationAttribute: [empEntity attributeNamed: @"department"]];

	myRelationship = [EORelationship new];
	[myRelationship setName: @"toEmployees"];
	[depEntity addRelationship: myRelationship];
	[myRelationship addJoin: myJoin];
	[myRelationship setJoinSemantic: EORightOuterJoin]; //what's this?
	[myRelationship setToMany: YES];

	RELEASE(myJoin);

	[depProps addObject: myRelationship];
	[depEntity setClassProperties: depProps];

	RELEASE(myRelationship);
	RELEASE(depProps);
	RELEASE(depEntity);

	[myModel writeToFile: @"enterprise"];

	EOModelGroup *mymg = [EOModelGroup new];
	[mymg addModel: myModel];
	[EOModelGroup setDefaultGroup: mymg];
	RELEASE(myModel);
	RELEASE(mymg);

	EOEditingContext *ec = [EOEditingContext new];
	EOFetchSpecification *fs = [EOFetchSpecification fetchSpecificationWithEntityName: @"Employees" qualifier: nil sortOrderings: nil];
	NSArray *ar = [ec objectsWithFetchSpecification: fs];
	NSLog(@"%@", ar);
	
}


int main(int argc, const char *argv[]) {
  id pool = [[NSAutoreleasePool alloc] init];

  // Your code here...
  gdl2test();

  // The end...
  [pool release];

  exit(0);
  return 0;
}
